package com.app.web.jsf.component;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.faces.render.Renderer;

import com.app.web.jsf.utils.JsfUtils;


/**
 * Render para pintar el componente FormBeanComponent.
 */
public class HtmlFormBeanRenderer extends Renderer {

	
	/**
	 * Se encarga de sacar de la request los valores que coinciden con las propiesdades 
	 * del objeto guardado en el atributo <tt>bean</tt>. Por introspección busca las propiedades del objeto, e intenta
	 * buscar un parámetro en la request con el mismo nombre.
	 * Sólo se considera propiedad cuando existe método get y set públicos, y el typo de retorno del get coincide con el
	 * tipo del único parámetro de entrada del set.
	 * Este componente es sólo un ejemplo y es válido sólo para las propieades de tipo String.
	 * 
	 * @see javax.faces.render.Renderer#decode(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	@Override
	public void decode(FacesContext facesContext, UIComponent uiComponent) {
		final FormBeanComponent formBeanComponent = (FormBeanComponent)uiComponent;
		
		// Se va a usar el Object asociado al componente para buscar las propiedades,
		// pero los valores recogidos de la request se almacenarar en la variable local.
		final Object componentValue = formBeanComponent.getBean();
		final Map<String, String> localValues = formBeanComponent.getLocalValues();
		localValues.clear();
		
		final Class<?> clazz = componentValue.getClass();
		final Method[] methods = clazz.getMethods();
		for (Method method : methods) {
			final String methodName = method.getName();
			final String propertyName = methodName.substring(3);
			
			if (method.getParameterTypes().length == 0 && method.getReturnType() == String.class && methodName.startsWith("get")) {
				// Sabemos que es un geter, ahora hay que comprobar si existe el setter equivalente
				final Class[] parameterTypes = { method.getReturnType() };
				final Method setterMethod;
				try {
					setterMethod = clazz.getMethod("set" + propertyName, parameterTypes);
				} catch (Exception e) {
					continue;	// no existe setter equivalente, así que seguimos buscando propiedades.
				}
				if (setterMethod.getReturnType() != Void.TYPE) {
					continue; // no es realmente un setter ya que un setter no debería tener tipo de retorno.
				}
				
				// Se busca en la request el nombre de la propiedad
				final String requestParameterName = propertyName.substring(0,1).toLowerCase() + propertyName.substring(1);
				final String requestParameter = (String)JsfUtils.getRequestParametersMap().get(requestParameterName);
				
				localValues.put(requestParameterName, requestParameter);
			}
		}
	}

	/**
	 * Se encarga de pintar los elementos del componente.
	 * Cuando el componente no puede tener componentes anidados, todo se pinta en el método <tt>encodeEnd</tt>. 
	 * 
	 * @see javax.faces.render.Renderer#encodeEnd(javax.faces.context.FacesContext, javax.faces.component.UIComponent)
	 */
	@Override
	public void encodeEnd(FacesContext facesContext, UIComponent uiComponent) throws IOException {
		final ResponseWriter out = facesContext.getResponseWriter();
		final FormBeanComponent formBeanComponent = (FormBeanComponent)uiComponent;
		final Object bean = formBeanComponent.getBean();
		final Class<?> clazz = bean.getClass();
		final Object[] getterArguments = {}; 

		out.startElement("table", null);
		for (Method method : clazz.getMethods()) {
			final String methodName = method.getName();
			final String propertyName = methodName.substring(3);
			
			if (method.getParameterTypes().length == 0 && method.getReturnType() == String.class && methodName.startsWith("get")) {
				// Sabemos que es un geter, ahora hay que comprobar si existe el setter equivalente
				Class[] parameterTypes = { method.getReturnType() };
				final Method setterMethod;
				try {
					setterMethod = clazz.getMethod("set" + propertyName, parameterTypes);
				} catch (Exception e) {
					continue;	// no existe setter equivalente, así que seguimos buscando propiedades.
				}
				final Class setterReturnType = setterMethod.getReturnType();
				if (setterReturnType != Void.TYPE) {
					continue; // no es realmente un setter ya que un setter no debería tener tipo de retorno.
				}
				
				// Se vuelca en la salida el campo de entrada para la propiedad
				final String outParameterName = propertyName.substring(0,1).toLowerCase() + propertyName.substring(1);
				out.startElement("tr", null);
				out.startElement("td", null);
				out.startElement("label", null);
				out.writeAttribute("for", outParameterName, null);
				out.writeText(outParameterName, null);
				out.endElement("label");
				out.endElement("td");
				out.startElement("td", null);
				out.startElement("input", null);
				out.writeAttribute("type", "text", null);
				out.writeAttribute("name", outParameterName, null);
				try {
					final String value = (String)method.invoke(bean, getterArguments);
					out.writeAttribute("value", value, null);
				} catch (Exception e) {
					throw new IOException(e);
				}
				out.endElement("input");
				out.endElement("td");
				out.endElement("tr");
			}
		}
		out.endElement("table");
	}

}
